# HG changeset patch
# User Sumalatha Polureddy
# Date 1436764003 -19800
# Node ID 839630920fe9313a78245fc34d468ed2d3c3e65b
# Parent  dd6945c3044e444b2a55261e3f074a4c10f14539
asm: testbench for costCoeffRemain()

diff -r dd6945c3044e -r 839630920fe9 source/common/x86/asm-primitives.cpp
--- a/source/common/x86/asm-primitives.cpp	Tue Jul 07 12:29:32 2015 +0530
+++ b/source/common/x86/asm-primitives.cpp	Mon Jul 13 10:36:43 2015 +0530
@@ -2510,7 +2510,7 @@
         p.costCoeffNxN = PFX(costCoeffNxN_sse4);
 #endif
         // TODO: it is passed smoke test, but we need testbench to active it, so temporary disable
-        //p.costCoeffRemain = x265_costCoeffRemain_sse4;
+        p.costCoeffRemain = x265_costCoeffRemain_sse4;
     }
     if (cpuMask & X265_CPU_AVX)
     {
diff -r dd6945c3044e -r 839630920fe9 source/test/pixelharness.cpp
--- a/source/test/pixelharness.cpp	Tue Jul 07 12:29:32 2015 +0530
+++ b/source/test/pixelharness.cpp	Mon Jul 13 10:36:43 2015 +0530
@@ -1699,6 +1699,37 @@
     return true;
 }
 
+bool PixelHarness::check_costCoeffRemain(costCoeffRemain_t ref, costCoeffRemain_t opt)
+{
+    ALIGN_VAR_32(uint16_t, absCoeff[1 << MLS_CG_SIZE]);
+
+    for (int i = 0; i < (1 << MLS_CG_SIZE); i++)
+    {
+        absCoeff[i] = rand() & SHORT_MAX;
+        // more zero coeff
zero?

+        if (absCoeff[i] < SHORT_MAX * 2 / 3)
+            absCoeff[i] = 1;
+    }
+    for (int i = 0; i < ITERS; i++)
+    {
+        uint32_t firstC2Idx = 0;
+        int numNonZero = rand() % 17; //can be random, range[1, 16]
+        for (int k = 0; k < C1FLAG_NUMBER; k++)
+        {
+            if (absCoeff[k] > 2)
+            {
+                firstC2Idx = absCoeff[k]; // it is exact first coeff that value more than 2
index?

+                break;
+            }
+        }
+        int ref_sum = ref(absCoeff, numNonZero, firstC2Idx);
+        int opt_sum = (int)checked(opt, absCoeff, numNonZero, firstC2Idx);
+        if (ref_sum != opt_sum)
+            return false;
+    }
+    return true;
+}
+
 bool PixelHarness::testPU(int part, const EncoderPrimitives& ref, const EncoderPrimitives& opt)
 {
     if (opt.pu[part].satd)
@@ -2309,6 +2340,14 @@
             return false;
         }
     }
+    if (opt.costCoeffRemain)
+    {
+        if (!check_costCoeffRemain(ref.costCoeffRemain, opt.costCoeffRemain))
+        {
+            printf("costCoeffRemain failed!\n");
+            return false;
+        }
+    }
 
     return true;
 }
@@ -2771,4 +2810,12 @@
 
         REPORT_SPEEDUP(opt.costCoeffNxN, ref.costCoeffNxN, g_scan4x4[SCAN_DIAG], coefBuf, 32, tmpOut, ctxSig, 0xFFFF, ctx, 1, 15, 32);
     }
+    if (opt.costCoeffRemain)
+    {
+        HEADER0("costCoeffRemain");
+        uint16_t abscoefBuf[32 * 32];
+        memset(abscoefBuf, 0, sizeof(abscoefBuf));
+        memset(abscoefBuf + 32 * 31, 1, 32 * sizeof(uint16_t));
+        REPORT_SPEEDUP(opt.costCoeffRemain, ref.costCoeffRemain, abscoefBuf, 16, 32);
total 16 coeffs, and first coeff is 32?


+    }
 }
diff -r dd6945c3044e -r 839630920fe9 source/test/pixelharness.h
--- a/source/test/pixelharness.h	Tue Jul 07 12:29:32 2015 +0530
+++ b/source/test/pixelharness.h	Mon Jul 13 10:36:43 2015 +0530
@@ -114,6 +114,7 @@
     bool check_scanPosLast(scanPosLast_t ref, scanPosLast_t opt);
     bool check_findPosFirstLast(findPosFirstLast_t ref, findPosFirstLast_t opt);
     bool check_costCoeffNxN(costCoeffNxN_t ref, costCoeffNxN_t opt);
+    bool check_costCoeffRemain(costCoeffRemain_t ref, costCoeffRemain_t opt);
 
 
 public:
